@@ -21,8 +21,8 @@ from webcode_tk import font_tools as fonts
 from webcode_tk import html_tools
 
 
-default_global_color = "#FFFFFF"
-default_global_background = None
+default_global_color = "#000000"
+default_global_background = "#ffffff"
 default_link_color = "#0000EE"
 default_link_visited = "#551A8B"
 root_font_size = 16
@@ -111,6 +111,13 @@ class Element(object):
             "specificity": "",
             "applied_by": "default",
         }
+        self.visited_background = {
+            "value": "",
+            "sheet": "",
+            "selector": "",
+            "specificity": "",
+            "applied_by": "default",
+        }
         self.hover_color = {
             "value": "",
             "sheet": "",
@@ -134,6 +141,13 @@ class Element(object):
             "large_aaa": True,
             "large_aa": True,
         }
+        self.hover_contrast_data = {
+            "ratio": 21.0,
+            "normal_aaa": True,
+            "normal_aa": True,
+            "large_aaa": True,
+            "large_aa": True,
+        }
         self.visited_contrast_data = {
             "ratio": 0.0,
             "normal_aaa": False,
@@ -156,6 +170,10 @@ class Element(object):
             self.visited_color["value"] = default_link_visited
             self.visited_color["sheet"] = "user-agent"
             self.visited_color["specificity"] = "000"
+            self.visited_background["value"] = default_global_background
+            self.visited_background["sheet"] = "user-agent"
+            self.visited_background["specificity"] = "000"
+            self.visited_background["applied_by"] = "context"
 
     def ammend_color_styles(self, new_styles: dict, filename: str) -> None:
         """adjust color styles based on specificity.
@@ -202,6 +220,25 @@ class Element(object):
                 # get contrast for color & bg_color
                 self.get_contrast_data("standard")
         else:
+            # be sure to change at least the background color for pseudo-states
+            if targets_bg_color:
+                # check hover background
+                hover_bg = self.hover_background.get("value")
+                hover_specificity = self.hover_background.get("specificity")
+                new_bg = new_styles.get("background-color")
+                if not hover_bg or new_specificity >= hover_specificity:
+                    self.hover_background["value"] = new_bg
+                    self.hover_background["specificity"] = new_specificity
+
+                # check visited background
+                visited_bg = self.visited_background.get("value")
+                visited_specificity = self.visited_background.get(
+                    "specificity"
+                )
+                if not visited_bg or new_specificity >= visited_specificity:
+                    self.visited_background["value"] = new_bg
+                    self.visited_background["specificity"] = new_specificity
+
             # Do we have a link selector?
             is_link_selector = css.is_link_selector(selector)
             if is_link_selector:
@@ -247,6 +284,11 @@ class Element(object):
             # If it was not directly set, we can change it
             if not bg_was_directly_set:
                 self.background_color["value"] = bg_color
+                if self.children:
+                    for kid in self.children:
+                        kid.ammend_bg_color(
+                            bg_color, new_selector, new_specificity, filename
+                        )
 
     def get_contrast_data(self, type="default") -> None:
         """get color contrast for the type specified
@@ -615,11 +657,17 @@ class Element(object):
                 )
                 self.background_color["gradient_colors"] = colors
             # set color, specificity, and directly applied
-            self.background_color["value"] = val
-            self.background_color["sheet"] = filename
-            self.background_color["specificity"] = new_specificity
-            self.background_color["applied_by"] = "directly"
-            self.background_color["selector"] = selector
+            if ":hover" in selector:
+                background_color_dict = self.hover_background
+            elif ":visited" in selector:
+                background_color_dict = self.visited_background
+            else:
+                background_color_dict = self.background_color
+            background_color_dict["value"] = val
+            background_color_dict["sheet"] = filename
+            background_color_dict["specificity"] = new_specificity
+            background_color_dict["applied_by"] = "directly"
+            background_color_dict["selector"] = selector
         else:
             if not previously_set:
                 self.background_color["value"] = val
@@ -678,6 +726,8 @@ class CSSAppliedTree:
 
     def __get_filename(self):
         """Extracts filename from path"""
+        if isinstance(self.file_path, dict):
+            self.file_path = self.file_path.get("file")
         self.filename = clerk.get_file_name(self.file_path)
 
     def __get_font_rules(self):
@@ -808,7 +858,14 @@ class CSSAppliedTree:
             for ruleset in color_rules:
                 # get filename
                 filename = clerk.get_file_name(sheet.href)
-                self.__adjust_colors(body, ruleset, filename)
+                selector_source = ""
+                if body.name == "body":
+                    selector_source = "body"
+                else:
+                    input("Wait, what?")
+                self.__adjust_colors(
+                    body, ruleset, filename, selector_src=selector_source
+                )
 
     def targets_body(self, ruleset: dict) -> bool:
         """returns whether the ruleset is targetting the body or not.
@@ -873,7 +930,12 @@ class CSSAppliedTree:
         return
 
     def __adjust_colors(
-        self, element: Element, ruleset: dict, filename: str
+        self,
+        element: Element,
+        ruleset: dict,
+        filename: str,
+        applied_by="default",
+        selector_src="",
     ) -> None:
         """recursively adjust color where necessary to all elements that
         apply.
@@ -924,6 +986,8 @@ class CSSAppliedTree:
             else:
                 if ":hover" in selector:
                     color = element.hover_color
+                elif ":visited" in selector:
+                    color = element.visited_color
                 else:
                     color = element.color
                 color["value"] = declaration.get("color")
@@ -932,8 +996,36 @@ class CSSAppliedTree:
                 color["specificity"] = specificity
                 color["applied_by"] = "directly"
             for child in element.children:
-                self.__adjust_child_colors(child, ruleset, filename)
+                element_name = element.name
+                self.__adjust_child_colors(
+                    child, ruleset, filename, element_name
+                )
+        elif is_background_prop:
+            # background - if not set directly - apply by context
+            value = declaration.get("background-color")
+            if not value:
+                value = declaration.get("background")
+            if property == "background-image":
+                value = declaration.get("background-image")
 
+            # adjust if value is not the same as current value
+            applied_by = element.background_color.get("applied_by")
+            current_value = element.background_color.get("value")
+            time_to_adjust = (
+                applied_by == "default"
+                or applied_by == "context"
+                and value != current_value
+            )
+            # But only if the value was applied to an ancestor
+            # For example, a th is not a parent of a td
+            applied_to_ancestor = False
+            for ancestor in element.ancestors:
+                ancestor_tag = ancestor[0]
+                if ancestor_tag == selector_src:
+                    applied_to_ancestor = True
+            time_to_adjust = time_to_adjust and applied_to_ancestor
+            if time_to_adjust:
+                print("Wait")
         can_inherit = self.can_inherit_styles(element, selector, ruleset)
         if can_inherit and applies:
             declaration = ruleset.get(selector)
@@ -945,7 +1037,11 @@ class CSSAppliedTree:
             self.__adjust_colors(child, ruleset, filename)
 
     def __adjust_child_colors(
-        self, child: Element, ruleset: dict, filename: str
+        self,
+        child: Element,
+        ruleset: dict,
+        filename: str,
+        applied_by="default",
     ) -> None:
         """adjusts colors for child of an element.
 
@@ -953,10 +1049,16 @@ class CSSAppliedTree:
         to see if the child element is a link or not (as well as the selector)
         and check the specificity of the ruleset.
 
+        To help with background colors, we need to know where it was applied
+        from (name of tag) - that way we can check if a declaration was applied
+        by one of the tag's parents. If it was, then we apply that color by
+        context taking the background from the nearest parent.
+
         Args:
             child: the child element in question
             ruleset: the CSS rule being applied
             filename: name of the file where the styles came from
+            applied_from: the name of the element that was targetted
         """
         # Adjust colors if necessary - if there was a change,
         # adjust colors for children
@@ -1155,9 +1257,19 @@ def does_selector_apply(element: Element, selector: str) -> bool:
 
         # If the tag is an anchor, but the selector is not - doesn't apply
         # link color can only be changed by a link selector
-        if element_name == "a" and not is_link_selector:
-            break
-        elif is_type_selector:
+        if element_name == "a":
+            # NOTE: turns out a descendant selector targetting an anchor won't
+            # show as a link_selector
+            if is_descendant_selector:
+                selector_list = selector.split()
+
+                # Check the descendant to see if it is a link
+                last_item = selector_list[-1]
+                if "a" != last_item[0]:
+                    break
+            elif not is_link_selector:
+                break
+        if is_type_selector:
             applies = element_name == sel
             if applies:
                 break
@@ -1493,7 +1605,7 @@ def update_contrast_results(
 
 
 def get_color_contrast_details(tree: CSSAppliedTree, rating="AAA") -> list:
-    """returns a list of all all failures or a pass based on rating
+    """returns a list of all failures or a pass based on rating
 
     We'll create a recursive inner nested function to traverse the DOM. We will
     target only the styles that are applied directly so as to not overwhelm the
@@ -1504,7 +1616,8 @@ def get_color_contrast_details(tree: CSSAppliedTree, rating="AAA") -> list:
         rating: the rating of contrast we check for (AAA or AA). The default
             is AAA.
     Returns:
-        results"""
+        results
+    """
     results = []
     filename = tree.filename
     # Loop recursively through the tree
@@ -1547,6 +1660,7 @@ def get_color_contrast_details(tree: CSSAppliedTree, rating="AAA") -> list:
                     passes = contrast_data.get("normal_aaa")
                 else:
                     passes = contrast_data.get("normal_aa")
+
             # only add results if failure
             if not passes:
                 if el.is_large:
@@ -1556,9 +1670,10 @@ def get_color_contrast_details(tree: CSSAppliedTree, rating="AAA") -> list:
                 selector = el.background_color.get("selector")
                 if not selector:
                     selector = el.color.get("selector")
-                msg = f"{el.name} tag (selector: {selector}) fails color "
-                msg += f"contrast at {size} {rating}."
+                msg = f"selector: {selector} fails color contrast {size}"
+                msg += f"{rating}"
                 results.append(msg)
+
         if not children:
             return
         else:
@@ -1567,14 +1682,45 @@ def get_color_contrast_details(tree: CSSAppliedTree, rating="AAA") -> list:
 
     for element in elements:
         check_element(element, rating, filename)
-    if not results:
+    if results:
+        adjusted = []
+        for result in results:
+            adjusted.append(f"fail: ({filename}) " + result)
+        results = adjusted
+    else:
         # It's a success! Let's let them know
-        msg = f"success: {filename} passes color contrast for {rating} Normal"
+        msg = f"pass: {filename} passes color contrast for {rating} Normal"
         msg += " and Large."
         results.append(msg)
     return results
 
 
+def get_color_contrast_report(dir_path: str, rating="AAA") -> list:
+    """returns a list of all failures or a pass based on rating
+
+    Args:
+        dir_path: the path to the directory in question
+        rating: the rating of contrast we check for (AAA or AA). The default
+            is AAA.
+    Returns:
+        results: a list of pass or fail messages.
+    """
+    results = []
+    files = css.get_styles_by_html_files(dir_path)
+    for file in files:
+        filepath = file.get("file")
+        stylesheets = file.get("stylesheets")
+        tree = CSSAppliedTree(filepath, stylesheets)
+
+        report = get_color_contrast_details(tree, rating)
+        for item in report:
+            if item not in results:
+                results.append(item)
+        # Do one more pass on hyperlinks and double check contrast
+        print(results)
+    return results
+
+
 if __name__ == "__main__":
     project_path = "tests/test_files/attribute_selector_file"
     project_path = "tests/test_files/large_project/"
